@@ -19,13 +19,13 @@ import stat
 from urllib.parse import quote as urlquote
 
 from twisted.internet import defer
-from twisted.internet import utils
 from twisted.python import log
 
 from buildbot import config
 from buildbot.changes import base
 from buildbot.util import bytes2unicode
 from buildbot.util import private_tempdir
+from buildbot.util import runprocess
 from buildbot.util.git import GitMixin
 from buildbot.util.git import getSshKnownHostsContents
 from buildbot.util.misc import writeLocalFile
@@ -37,7 +37,7 @@ class GitError(Exception):
     """Raised when git exits with code 128."""
 
 
-class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
+class GitPoller(base.ReconfigurablePollingChangeSource, StateMixin, GitMixin):
 
     """This source will poll a remote git repo for changes and submit
     them to the change master."""
@@ -49,11 +49,52 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
 
     secrets = ("sshPrivateKey", "sshHostKey", "sshKnownHosts")
 
-    def __init__(self, repourl, branches=None, branch=None, workdir=None, pollInterval=10 * 60,
-                 gitbin="git", usetimestamps=True, category=None, project=None, pollinterval=-2,
-                 fetch_refspec=None, encoding="utf-8", name=None, pollAtLaunch=False,
-                 buildPushesWithNoCommits=False, only_tags=False, sshPrivateKey=None,
-                 sshHostKey=None, sshKnownHosts=None, pollRandomDelayMin=0, pollRandomDelayMax=0):
+    def __init__(self, repourl, **kwargs):
+        name = kwargs.get("name", None)
+        if name is None:
+            kwargs["name"] = repourl
+        super().__init__(repourl, **kwargs)
+
+    def checkConfig(self, repourl, branches=None, branch=None, workdir=None,
+                    pollInterval=10 * 60, gitbin="git", usetimestamps=True, category=None,
+                    project=None, pollinterval=-2, fetch_refspec=None, encoding="utf-8",
+                    name=None, pollAtLaunch=False, buildPushesWithNoCommits=False,
+                    only_tags=False, sshPrivateKey=None, sshHostKey=None, sshKnownHosts=None,
+                    pollRandomDelayMin=0, pollRandomDelayMax=0):
+
+        # for backward compatibility; the parameter used to be spelled with 'i'
+        if pollinterval != -2:
+            pollInterval = pollinterval
+
+        if only_tags and (branch or branches):
+            config.error("GitPoller: can't specify only_tags and branch/branches")
+        if branch and branches:
+            config.error("GitPoller: can't specify both branch and branches")
+
+        self.sshPrivateKey = sshPrivateKey
+        self.sshHostKey = sshHostKey
+        self.sshKnownHosts = sshKnownHosts
+        self.setupGit(logname='GitPoller')  # check the configuration
+
+        if fetch_refspec is not None:
+            config.error("GitPoller: fetch_refspec is no longer supported. "
+                         "Instead, only the given branches are downloaded.")
+
+        if name is None:
+            name = repourl
+
+        super().checkConfig(name=name,
+                            pollInterval=pollInterval, pollAtLaunch=pollAtLaunch,
+                            pollRandomDelayMin=pollRandomDelayMin,
+                            pollRandomDelayMax=pollRandomDelayMax)
+
+    @defer.inlineCallbacks
+    def reconfigService(self, repourl, branches=None, branch=None, workdir=None,
+                        pollInterval=10 * 60, gitbin="git", usetimestamps=True, category=None,
+                        project=None, pollinterval=-2, fetch_refspec=None, encoding="utf-8",
+                        name=None, pollAtLaunch=False, buildPushesWithNoCommits=False,
+                        only_tags=False, sshPrivateKey=None, sshHostKey=None, sshKnownHosts=None,
+                        pollRandomDelayMin=0, pollRandomDelayMax=0):
 
         # for backward compatibility; the parameter used to be spelled with 'i'
         if pollinterval != -2:
@@ -62,19 +103,10 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         if name is None:
             name = repourl
 
-        super().__init__(name=name, pollInterval=pollInterval, pollAtLaunch=pollAtLaunch,
-                         pollRandomDelayMin=pollRandomDelayMin,
-                         pollRandomDelayMax=pollRandomDelayMax, sshPrivateKey=sshPrivateKey,
-                         sshHostKey=sshHostKey, sshKnownHosts=sshKnownHosts)
-
         if project is None:
             project = ''
 
-        if only_tags and (branch or branches):
-            config.error("GitPoller: can't specify only_tags and branch/branches")
-        if branch and branches:
-            config.error("GitPoller: can't specify both branch and branches")
-        elif branch:
+        if branch:
             branches = [branch]
         elif not branches:
             if only_tags:
@@ -99,13 +131,20 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         self.sshKnownHosts = sshKnownHosts
         self.setupGit(logname='GitPoller')
 
-        if fetch_refspec is not None:
-            config.error("GitPoller: fetch_refspec is no longer supported. "
-                         "Instead, only the given branches are downloaded.")
-
         if self.workdir is None:
             self.workdir = 'gitpoller-work'
 
+        # make our workdir absolute, relative to the master's basedir
+
+        if not os.path.isabs(self.workdir):
+            self.workdir = os.path.join(self.master.basedir, self.workdir)
+            log.msg("gitpoller: using workdir '{}'".format(self.workdir))
+
+        yield super().reconfigService(name=name,
+                                      pollInterval=pollInterval, pollAtLaunch=pollAtLaunch,
+                                      pollRandomDelayMin=pollRandomDelayMin,
+                                      pollRandomDelayMax=pollRandomDelayMax)
+
     @defer.inlineCallbacks
     def _checkGitFeatures(self):
         stdout = yield self._dovccmd('--version', [])
@@ -120,11 +159,6 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
 
     @defer.inlineCallbacks
     def activate(self):
-        # make our workdir absolute, relative to the master's basedir
-        if not os.path.isabs(self.workdir):
-            self.workdir = os.path.join(self.master.basedir, self.workdir)
-            log.msg("gitpoller: using workdir '{}'".format(self.workdir))
-
         try:
             self.lastRev = yield self.getState('lastRev', {})
 
@@ -177,6 +211,11 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         url = urlquote(self.repourl, '').replace('~', '%7E')
         return "refs/buildbot/{}/{}".format(url, self._removeHeads(branch))
 
+    def poll_should_exit(self):
+        # A single gitpoller loop may take a while on a loaded master, which would block
+        # reconfiguration, so we try to exit early.
+        return not self.doPoll.running
+
     @defer.inlineCallbacks
     def poll(self):
         yield self._checkGitFeatures()
@@ -189,6 +228,10 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
 
         branches = self.branches if self.branches else []
         remote_refs = yield self._getBranches()
+
+        if self.poll_should_exit():
+            return
+
         if branches is True or callable(branches):
             if callable(self.branches):
                 branches = [b for b in remote_refs if self.branches(b)]
@@ -214,6 +257,11 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
         log.msg('gitpoller: processing changes from "{}"'.format(self.repourl))
         for branch in branches:
             try:
+                if self.poll_should_exit():  # pragma: no cover
+                    # Note that we still want to update the last known revisions for the branches
+                    # we did process
+                    break
+
                 rev = yield self._dovccmd(
                     'rev-parse', [self._trackerBranch(branch)], path=self.workdir)
                 revs[branch] = bytes2unicode(rev, self.encoding)
@@ -222,7 +270,7 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
                 log.err(_why="trying to poll branch {} of {}".format(
                         branch, self.repourl))
 
-        self.lastRev.update(revs)
+        self.lastRev = revs
         yield self.setState('lastRev', self.lastRev)
 
     def _get_commit_comments(self, rev):
@@ -425,9 +473,9 @@ class GitPoller(base.PollingChangeSource, StateMixin, GitMixin):
 
         full_args += [command] + args
 
-        res = yield utils.getProcessOutputAndValue(self.gitbin,
-            full_args, path=path, env=full_env)
-        (stdout, stderr, code) = res
+        res = yield runprocess.run_process(self.master.reactor, [self.gitbin] + full_args, path,
+                                           env=full_env)
+        (code, stdout, stderr) = res
         stdout = bytes2unicode(stdout, self.encoding)
         stderr = bytes2unicode(stderr, self.encoding)
         if code != 0:
