@@ -365,11 +365,11 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
         self.reconfig_active = self.reactor.seconds()
         metrics.MetricCountEvent.log("loaded_config", 1)
 
-        # notify every 10 seconds that the reconfig is still going on, although
-        # reconfigs should not take that long!
+        # notify every 10 seconds that the reconfig is still going on, the duration of reconfigs is
+        # longer on larger installations and may take a while.
         self.reconfig_notifier = task.LoopingCall(
-            lambda: log.msg("reconfig is ongoing for {} s".format(self.reactor.seconds() -
-                                                                  self.reconfig_active)))
+            lambda: log.msg("reconfig is ongoing for {:.3f} s".format(self.reactor.seconds() -
+                                                                      self.reconfig_active)))
         self.reconfig_notifier.start(10, now=False)
 
         timer = metrics.Timer("BuildMaster.reconfig")
@@ -391,6 +391,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
     @defer.inlineCallbacks
     def doReconfig(self):
         log.msg("beginning configuration update")
+        time_started = self.reactor.seconds()
         changes_made = False
         failed = False
         try:
@@ -419,12 +420,13 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
         if failed:
             if changes_made:
-                log.msg("WARNING: reconfig partially applied; master "
-                        "may malfunction")
+                msg = "WARNING: configuration update partially applied; master may malfunction"
             else:
-                log.msg("reconfig aborted without making any changes")
+                msg = "configuration update aborted without making any changes"
         else:
-            log.msg("configuration update complete")
+            msg = "configuration update complete"
+
+        log.msg("{} (took {:.3f} seconds)".format(msg, self.reactor.seconds() - time_started))
 
     def reconfigServiceWithBuildbotConfig(self, new_config):
         if self.configured_db_url is None:
