@@ -51,8 +51,8 @@ def validateMasterOption(master):
     try:
         hostname, port = master.split(":")
         port = int(port)
-    except (TypeError, ValueError):
-        raise usage.UsageError("master must have the form 'hostname:port'")
+    except (TypeError, ValueError) as e:
+        raise usage.UsageError("master must have the form 'hostname:port'") from e
 
 
 class UpgradeMasterOptions(base.BasedirMixin, base.SubcommandOptions):
@@ -155,16 +155,16 @@ class CreateMasterOptions(base.BasedirMixin, base.SubcommandOptions):
         else:
             try:
                 self['log-count'] = int(self['log-count'])
-            except ValueError:
+            except ValueError as e:
                 raise usage.UsageError(
-                    "log-count parameter needs to be an int or None")
+                    "log-count parameter needs to be an int or None") from e
 
         # validate 'db' parameter
         try:
             # check if sqlalchemy will be able to parse specified URL
             sa.engine.url.make_url(self['db'])
-        except sa.exc.ArgumentError:
-            raise usage.UsageError("could not parse database URL '{}'".format(self['db']))
+        except sa.exc.ArgumentError as e:
+            raise usage.UsageError("could not parse database URL '{}'".format(self['db'])) from e
 
 
 class StopOptions(base.BasedirMixin, base.SubcommandOptions):
@@ -218,6 +218,11 @@ class ReconfigOptions(base.BasedirMixin, base.SubcommandOptions):
         ['quiet', 'q', "Don't display log messages about reconfiguration"],
     ]
 
+    optParameters = [
+        ['progress_timeout', None, None,
+         'The amount of time the script waits for messages in the logs that indicate progress.'],
+    ]
+
     def getSynopsis(self):
         return "Usage:    buildbot reconfig [<basedir>]"
 
@@ -286,8 +291,8 @@ class SendChangeOptions(base.SubcommandOptions):
         if self.get('when'):
             try:
                 self['when'] = float(self['when'])
-            except (TypeError, ValueError):
-                raise usage.UsageError('invalid "when" value {}'.format(self['when']))
+            except (TypeError, ValueError) as e:
+                raise usage.UsageError('invalid "when" value {}'.format(self['when'])) from e
         else:
             self['when'] = None
 
@@ -651,6 +656,16 @@ class DataSpecOption(base.BasedirMixin, base.SubcommandOptions):
         return "Usage:   buildbot dataspec [options]"
 
 
+class GenGraphQLOption(base.BasedirMixin, base.SubcommandOptions):
+    subcommandFunction = "buildbot.scripts.gengraphql.gengraphql"
+    optParameters = [
+        ['out', 'o', "graphql.schema", "output to specified path"],
+    ]
+
+    def getSynopsis(self):
+        return "Usage:   buildbot graphql-schema [options]"
+
+
 class DevProxyOptions(base.BasedirMixin, base.SubcommandOptions):
 
     """Run a fake web server serving the local ui frontend and a distant rest and websocket api.
@@ -737,6 +752,8 @@ class Options(usage.Options):
         ['dev-proxy', None, DevProxyOptions,
          "Run a fake web server serving the local ui frontend and a distant rest and websocket api."
          ],
+        ['graphql-schema', None, GenGraphQLOption,
+         "Output graphql api schema"],
         ['cleanupdb', None, CleanupDBOptions,
          "cleanup the database"
          ]
