@@ -60,23 +60,34 @@ class HashiCorpVaultSecretProvider(SecretProviderBase):
         """
         get the value from vault secret backend
         """
-        if self.apiVersion == 1:
-            path = self.secretsmount + '/' + entry
+        parts = entry.rsplit('/', maxsplit=1)
+        name = parts[0]
+        if len(parts) > 1:
+            key = parts[1]
         else:
-            path = self.secretsmount + '/data/' + entry
+            key = 'value'
+
+        if self.apiVersion == 1:
+            path = self.secretsmount + '/' + name
+        else:
+            path = self.secretsmount + '/data/' + name
 
         # note that the HTTP path contains v1 for both versions of the key-value
         # secret engine. Different versions of the key-value engine are
         # effectively separate secret engines in vault, with the same base HTTP
         # API, but with different paths within it.
-        proj = yield self._http.get('/v1/{0}'.format(path))
+        proj = yield self._http.get(f"/v1/{path}")
         code = yield proj.code
         if code != 200:
-            raise KeyError(("The key {} does not exist in Vault provider: request"
-                           " return code:{}.").format(entry, code))
+            raise KeyError(("The secret {} does not exist in Vault provider: request"
+                           " return code: {}.").format(entry, code))
         json = yield proj.json()
         if self.apiVersion == 1:
-            ret = json.get('data', {}).get('value')
+            secrets = json.get('data', {})
         else:
-            ret = json.get('data', {}).get('data', {}).get('value')
-        return ret
+            secrets = json.get('data', {}).get('data', {})
+        try:
+            return secrets[key]
+        except KeyError as e:
+            raise KeyError(
+                "The secret {} does not exist in Vault provider: {}".format(entry, e)) from e
