@@ -22,7 +22,7 @@ import tarfile
 
 import migrate
 import migrate.versioning.api
-from sqlalchemy.engine import reflection
+import sqlalchemy as sa
 from sqlalchemy.exc import DatabaseError
 
 from twisted.internet import defer
@@ -127,7 +127,7 @@ class UpgradeTestMixin(db.RealDatabaseMixin, TestReactorMixin):
                 return diff
 
             # check indexes manually
-            insp = reflection.Inspector.from_engine(engine)
+            insp = sa.inspect(engine)
             # unique, name, column_names
             diff = []
             for tbl in self.db.model.metadata.sorted_tables:
@@ -215,11 +215,11 @@ class UpgradeTestEmpty(UpgradeTestMixin, unittest.TestCase):
         os_encoding = locale.getpreferredencoding()
         try:
             '\N{SNOWMAN}'.encode(os_encoding)
-        except UnicodeEncodeError:
+        except UnicodeEncodeError as e:
             # Default encoding of Windows console is 'cp1252'
             # which cannot encode the snowman.
-            raise(unittest.SkipTest("Cannot encode weird unicode "
-                "on this platform with {}".format(os_encoding)))
+            raise unittest.SkipTest("Cannot encode weird unicode "
+                "on this platform with {}".format(os_encoding)) from e
 
         d = self.db.model.upgrade()
         d.addCallback(lambda r: self.assertModelMatches())
