@@ -16,6 +16,7 @@
 import json
 import os
 import pkg_resources
+from io import BytesIO
 from io import StringIO
 from urllib.parse import parse_qs
 from urllib.parse import unquote as urlunquote
@@ -157,7 +158,8 @@ class WwwTestMixin(RequiresWwwMixin):
 
     def render_resource(self, rsrc, path=b'/', accept=None, method=b'GET',
                         origin=None, access_control_request_method=None,
-                        extraHeaders=None, request=None):
+                        extraHeaders=None, request=None,
+                        content=None, content_type=None):
         if not request:
             request = self.make_request(path, method=method)
             if accept:
@@ -169,6 +171,9 @@ class WwwTestMixin(RequiresWwwMixin):
                     access_control_request_method
             if extraHeaders is not None:
                 request.input_headers.update(extraHeaders)
+            if content_type is not None:
+                request.input_headers.update({b'content-type': content_type})
+                request.content = BytesIO(content)
 
         rv = rsrc.render(request)
         if rv != server.NOT_DONE_YET:
