@@ -17,6 +17,7 @@ from twisted.internet import defer
 
 from buildbot.util import service
 from buildbot.util import subscription
+from buildbot.util.eventual import eventually
 
 
 class Listener(service.ReconfigurableServiceMixin, service.AsyncMultiService):
@@ -26,10 +27,7 @@ class Listener(service.ReconfigurableServiceMixin, service.AsyncMultiService):
 class Connection:
     proxies = {}
 
-    def __init__(self, master, worker):
-        self.master = master
-        self.worker = worker
-        name = worker.workername
+    def __init__(self, name):
         self._disconnectSubs = subscription.SubscriptionPoint("disconnections from {}".format(name))
 
     # This method replace all Impl args by their Proxy protocol implementation
@@ -41,17 +39,20 @@ class Connection:
                     v = proxyclass(v)
             newargs[k] = v
         return newargs
+
     # disconnection handling
 
+    def wait_shutdown_started(self):
+        d = defer.Deferred()
+        self.notifyOnDisconnect(lambda: eventually(d.callback, None))
+        return d
+
     def waitShutdown(self):
-        return defer.succeed(None)
+        return self._disconnectSubs.waitForDeliveriesToFinish()
 
     def notifyOnDisconnect(self, cb):
         return self._disconnectSubs.subscribe(cb)
 
-    def waitForNotifyDisconnectedDelivered(self):
-        return self._disconnectSubs.waitForDeliveriesToFinish()
-
     def notifyDisconnected(self):
         self._disconnectSubs.deliver()
 
