@@ -132,7 +132,9 @@ class Connection(base.Connection, pb.Avatar):
     info = None
 
     def __init__(self, master, worker, mind):
-        super().__init__(master, worker)
+        super().__init__(worker.workername)
+        self.master = master
+        self.worker = worker
         self.mind = mind
         self._keepalive_waiter = deferwaiter.DeferWaiter()
         self._keepalive_action_handler = \
@@ -144,6 +146,7 @@ class Connection(base.Connection, pb.Avatar):
     @defer.inlineCallbacks
     def attached(self, mind):
         self.startKeepaliveTimer()
+        self.notifyOnDisconnect(self._stop_keepalive_timer)
         # pbmanager calls perspective.attached; pass this along to the
         # worker
         yield self.worker.attached(self)
@@ -157,7 +160,7 @@ class Connection(base.Connection, pb.Avatar):
 
     # disconnection handling
     @defer.inlineCallbacks
-    def waitShutdown(self):
+    def _stop_keepalive_timer(self):
         self.stopKeepaliveTimer()
         yield self._keepalive_waiter.wait()
 
