@@ -135,6 +135,8 @@ class WsProtocolFactory(WebSocketServerFactory):
     def __init__(self, master):
         super().__init__()
         self.master = master
+        pingInterval = self.master.config.www.get('ws_ping_interval', 0)
+        self.setProtocolOptions(webStatus=False, autoPingInterval=pingInterval)
 
     def buildProtocol(self, addr):
         p = WsProtocol(self.master)
